@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2015-2016 Mateusz Łącki and Michał Startek.
+ *   Copyright (C) 2015-2019 Mateusz Łącki and Michał Startek.
  *
  *   This file is part of IsoSpec.
  *
@@ -8,7 +8,7 @@
  *
  *   IsoSpec is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
  *   You should have received a copy of the Simplified BSD Licence
  *   along with IsoSpec.  If not, see <https://opensource.org/licenses/BSD-2-Clause>.
@@ -18,10 +18,11 @@
 #include <iostream>
 #include "allocator.h"
 
-
+namespace IsoSpec
+{
 
 template <typename T>
-Allocator<T>::Allocator(const int dim, const int tabSize): currentId(0), dim(dim), tabSize(tabSize)
+Allocator<T>::Allocator(const int dim, const int tabSize): currentId(-1), dim(dim), tabSize(tabSize)
 {
     currentTab = new T[dim * tabSize];
 }
@@ -46,3 +47,5 @@ void Allocator<T>::shiftTables()
 }
 
 template class Allocator<int>;
+
+}
