@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2015-2016 Mateusz Łącki and Michał Startek.
+ *   Copyright (C) 2015-2019 Mateusz Łącki and Michał Startek.
  *
  *   This file is part of IsoSpec.
  *
@@ -8,21 +8,22 @@
  *
  *   IsoSpec is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
  *   You should have received a copy of the Simplified BSD Licence
  *   along with IsoSpec.  If not, see <https://opensource.org/licenses/BSD-2-Clause>.
  */
 
 
-
 #include "misc.h"
-#include "lang.h"
+#include "platform.h"
 #include <stdlib.h>
 
 #define mswap(x, y) swapspace = x; x = y; y=swapspace;
 
 
+namespace IsoSpec
+{
 
 void* quickselect(void** array, int n, int start, int end)
 {
@@ -35,10 +36,10 @@ void* quickselect(void** array, int n, int start, int end)
     {
         // Partition part
         int len = end - start;
-#ifdef BUILDING_R
+#if ISOSPEC_BUILDING_R
         int pivot = len/2 + start;
 #else
-	int pivot = rand() % len + start;
+        int pivot = rand() % len + start;
 #endif
         void* pval = array[pivot];
         double pprob = getLProb(pval);
@@ -64,3 +65,5 @@ void* quickselect(void** array, int n, int start, int end)
     };
 }
 
+} // namespace IsoSpec
+
