@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2015-2016 Mateusz Łącki and Michał Startek.
+ *   Copyright (C) 2015-2019 Mateusz Łącki and Michał Startek.
  *
  *   This file is part of IsoSpec.
  *
@@ -8,16 +8,19 @@
  *
  *   IsoSpec is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
  *   You should have received a copy of the Simplified BSD Licence
  *   along with IsoSpec.  If not, see <https://opensource.org/licenses/BSD-2-Clause>.
  */
 
-#ifndef SUMMATOR_HPP
-#define SUMMATOR_HPP
+#pragma once
+
 #include <cmath>
 
+namespace IsoSpec
+{
+
 class SSummator
 {
     // Shewchuk algorithm
@@ -94,7 +97,7 @@ public:
 
 class TSummator
 {
-    // Tirival algorithm
+    // Trivial algorithm, for testing only
     double sum;
 public:
     inline TSummator()
@@ -106,9 +109,10 @@ public:
     }
     inline double get()
     {
-    	return sum;
+        return sum;
     }
 };
 
-#endif
+
+} // namespace IsoSpec
 
